@@ -10,7 +10,6 @@ Keras (native) format
 import importlib
 import os
 import yaml
-import gorilla
 import tempfile
 import shutil
 
@@ -19,6 +18,7 @@ import pandas as pd
 from distutils.version import LooseVersion
 from mlflow import pyfunc
 from mlflow.models import Model
+from mlflow.models.model import MLMODEL_FILE_NAME
 import mlflow.tracking
 from mlflow.exceptions import MlflowException
 from mlflow.models.signature import ModelSignature
@@ -27,15 +27,24 @@ from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.utils.environment import _mlflow_conda_env
 from mlflow.utils.model_utils import _get_flavor_configuration
 from mlflow.utils.annotations import experimental
-from mlflow.utils.autologging_utils import try_mlflow_log, log_fn_args_as_params
+from mlflow.utils.autologging_utils import (
+    autologging_integration,
+    safe_patch,
+    ExceptionSafeClass,
+    try_mlflow_log,
+    log_fn_args_as_params,
+    batch_metrics_logger,
+)
+from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 
 
 FLAVOR_NAME = "keras"
 # File name to which custom objects cloudpickle is saved - used during save and load
 _CUSTOM_OBJECTS_SAVE_PATH = "custom_objects.cloudpickle"
 _KERAS_MODULE_SPEC_PATH = "keras_module.txt"
+_KERAS_SAVE_FORMAT_PATH = "save_format.txt"
 # File name to which keras model is saved
-_MODEL_SAVE_PATH = "model.h5"
+_MODEL_SAVE_PATH = "model"
 # Conda env subpath when saving/loading model
 _CONDA_ENV_SUBPATH = "conda.yaml"
 
@@ -46,39 +55,55 @@ def get_default_conda_env(include_cloudpickle=False, keras_module=None):
              :func:`save_model()` and :func:`log_model()`.
     """
     import tensorflow as tf
+
     conda_deps = []  # if we use tf.keras we only need to declare dependency on tensorflow
     pip_deps = []
     if keras_module is None:
         import keras
+
         keras_module = keras
     if keras_module.__name__ == "keras":
         # Temporary fix: the created conda environment has issues installing keras >= 2.3.1
-        if LooseVersion(keras_module.__version__) < LooseVersion('2.3.1'):
+        if LooseVersion(keras_module.__version__) < LooseVersion("2.3.1"):
             conda_deps.append("keras=={}".format(keras_module.__version__))
         else:
             pip_deps.append("keras=={}".format(keras_module.__version__))
     if include_cloudpickle:
         import cloudpickle
+
         pip_deps.append("cloudpickle=={}".format(cloudpickle.__version__))
     # Temporary fix: conda-forge currently does not have tensorflow > 1.14
     # The Keras pyfunc representation requires the TensorFlow
     # backend for Keras. Therefore, the conda environment must
     # include TensorFlow
-    if LooseVersion(tf.__version__) <= LooseVersion('1.13.2'):
+    if LooseVersion(tf.__version__) <= LooseVersion("1.13.2"):
         conda_deps.append("tensorflow=={}".format(tf.__version__))
     else:
         pip_deps.append("tensorflow=={}".format(tf.__version__))
 
+    # Tensorflow<2.4 does not work with h5py>=3.0.0
+    # see https://github.com/tensorflow/tensorflow/issues/44467
+    if LooseVersion(tf.__version__) < LooseVersion("2.4"):
+        pip_deps.append("h5py<3.0.0")
+
     return _mlflow_conda_env(
         additional_conda_deps=conda_deps,
         additional_pip_deps=pip_deps,
-        additional_conda_channels=None)
+        additional_conda_channels=None,
+    )
 
 
-def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_objects=None,
-               keras_module=None,
-               signature: ModelSignature = None, input_example: ModelInputExample = None,
-               **kwargs):
+def save_model(
+    keras_model,
+    path,
+    conda_env=None,
+    mlflow_model=None,
+    custom_objects=None,
+    keras_module=None,
+    signature: ModelSignature = None,
+    input_example: ModelInputExample = None,
+    **kwargs
+):
     """
     Save a Keras model to a path on the local file system.
 
@@ -127,9 +152,10 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
                         signature = infer_signature(train, predictions)
     :param input_example: (Experimental) Input example provides one or several instances of valid
                           model input. The example can be used as a hint of what data to feed the
-                          model. The given example will be converted to a Pandas DataFrame and then
-                          serialized to json using the Pandas split-oriented format. Bytes are
-                          base64-encoded.
+                          model. The given example can be a Pandas DataFrame where the given
+                          example will be serialized to json using the Pandas split-oriented
+                          format, or a numpy array where the example will be serialized to json
+                          by converting it to a list. Bytes are base64-encoded.
 
     .. code-block:: python
         :caption: Example
@@ -145,11 +171,20 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
         mlflow.keras.save_model(keras_model, keras_model_path)
     """
     if keras_module is None:
+
         def _is_plain_keras(model):
             try:
-                # NB: Network is the first parent with save method
-                import keras.engine.network
-                return isinstance(model, keras.engine.network.Network)
+                import keras
+
+                if LooseVersion(keras.__version__) < LooseVersion("2.2.0"):
+                    import keras.engine
+
+                    return isinstance(model, keras.engine.Model)
+                else:
+                    # NB: Network is the first parent with save method
+                    import keras.engine.network
+
+                    return isinstance(model, keras.engine.network.Network)
             except ImportError:
                 return False
 
@@ -157,6 +192,7 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
             try:
                 # NB: Network is not exposed in tf.keras, we check for Model instead.
                 import tensorflow.keras.models
+
                 return isinstance(model, tensorflow.keras.models.Model)
             except ImportError:
                 return False
@@ -166,9 +202,11 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
         elif _is_tf_keras(keras_model):
             keras_module = importlib.import_module("tensorflow.keras")
         else:
-            raise MlflowException("Unable to infer keras module from the model, please specify "
-                                  "which keras module ('keras' or 'tensorflow.keras') is to be "
-                                  "used to save and load the model.")
+            raise MlflowException(
+                "Unable to infer keras module from the model, please specify "
+                "which keras module ('keras' or 'tensorflow.keras') is to be "
+                "used to save and load the model."
+            )
     elif type(keras_module) == str:
         keras_module = importlib.import_module(keras_module)
 
@@ -197,13 +235,24 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
     with open(os.path.join(data_path, _KERAS_MODULE_SPEC_PATH), "w") as f:
         f.write(keras_module.__name__)
 
-    # save keras model to path/data/model.h5
+    # Use the SavedModel format if `save_format` is unspecified
+    save_format = kwargs.get("save_format", "tf")
+
+    # save keras save_format to path/data/save_format.txt
+    with open(os.path.join(data_path, _KERAS_SAVE_FORMAT_PATH), "w") as f:
+        f.write(save_format)
+
+    # save keras model
+    # To maintain prior behavior, when the format is HDF5, we save
+    # with the h5 file extension. Otherwise, model_path is a directory
+    # where the saved_model.pb will be stored (for SavedModel format)
+    file_extension = ".h5" if save_format == "h5" else ""
     model_subpath = os.path.join(data_subpath, _MODEL_SAVE_PATH)
-    model_path = os.path.join(path, model_subpath)
-    if path.startswith('/dbfs/'):
+    model_path = os.path.join(path, model_subpath) + file_extension
+    if path.startswith("/dbfs/"):
         # The Databricks Filesystem uses a FUSE implementation that does not support
         # random writes. It causes an error.
-        with tempfile.NamedTemporaryFile(suffix='.h5') as f:
+        with tempfile.NamedTemporaryFile(suffix=".h5") as f:
             keras_model.save(f.name, **kwargs)
             f.flush()  # force flush the data
             shutil.copyfile(src=f.name, dst=model_path)
@@ -211,15 +260,19 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
         keras_model.save(model_path, **kwargs)
 
     # update flavor info to mlflow_model
-    mlflow_model.add_flavor(FLAVOR_NAME,
-                            keras_module=keras_module.__name__,
-                            keras_version=keras_module.__version__,
-                            data=data_subpath)
+    mlflow_model.add_flavor(
+        FLAVOR_NAME,
+        keras_module=keras_module.__name__,
+        keras_version=keras_module.__version__,
+        save_format=save_format,
+        data=data_subpath,
+    )
 
     # save conda.yaml info to path/conda.yml
     if conda_env is None:
-        conda_env = get_default_conda_env(include_cloudpickle=custom_objects is not None,
-                                          keras_module=keras_module)
+        conda_env = get_default_conda_env(
+            include_cloudpickle=custom_objects is not None, keras_module=keras_module
+        )
     elif not isinstance(conda_env, dict):
         with open(conda_env, "r") as f:
             conda_env = yaml.safe_load(f)
@@ -227,16 +280,26 @@ def save_model(keras_model, path, conda_env=None, mlflow_model=None, custom_obje
         yaml.safe_dump(conda_env, stream=f, default_flow_style=False)
 
     # append loader_module, data and env data to mlflow_model
-    pyfunc.add_to_model(mlflow_model, loader_module="mlflow.keras",
-                        data=data_subpath, env=_CONDA_ENV_SUBPATH)
+    pyfunc.add_to_model(
+        mlflow_model, loader_module="mlflow.keras", data=data_subpath, env=_CONDA_ENV_SUBPATH
+    )
 
     # save mlflow_model to path/MLmodel
-    mlflow_model.save(os.path.join(path, "MLmodel"))
+    mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
 
-def log_model(keras_model, artifact_path, conda_env=None, custom_objects=None, keras_module=None,
-              registered_model_name=None, signature: ModelSignature=None,
-              input_example: ModelInputExample=None, **kwargs):
+def log_model(
+    keras_model,
+    artifact_path,
+    conda_env=None,
+    custom_objects=None,
+    keras_module=None,
+    registered_model_name=None,
+    signature: ModelSignature = None,
+    input_example: ModelInputExample = None,
+    await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
+    **kwargs
+):
     """
     Log a Keras model as an MLflow artifact for the current run.
 
@@ -288,10 +351,13 @@ def log_model(keras_model, artifact_path, conda_env=None, custom_objects=None, k
                         signature = infer_signature(train, predictions)
     :param input_example: (Experimental) Input example provides one or several instances of valid
                           model input. The example can be used as a hint of what data to feed the
-                          model. The given example will be converted to a Pandas DataFrame and then
-                          serialized to json using the Pandas split-oriented format. Bytes are
-                          base64-encoded.
-
+                          model. The given example can be a Pandas DataFrame where the given
+                          example will be serialized to json using the Pandas split-oriented
+                          format, or a numpy array where the example will be serialized to json
+                          by converting it to a list. Bytes are base64-encoded.
+    :param await_registration_for: Number of seconds to wait for the model version to finish
+                            being created and is in ``READY`` status. By default, the function
+                            waits for five minutes. Specify 0 or None to skip waiting.
     :param kwargs: kwargs to pass to ``keras_model.save`` method.
 
     .. code-block:: python
@@ -308,11 +374,19 @@ def log_model(keras_model, artifact_path, conda_env=None, custom_objects=None, k
         with mlflow.start_run() as run:
             mlflow.keras.log_model(keras_model, "models")
     """
-    Model.log(artifact_path=artifact_path, flavor=mlflow.keras,
-              keras_model=keras_model, conda_env=conda_env, custom_objects=custom_objects,
-              keras_module=keras_module, registered_model_name=registered_model_name,
-              signature=signature, input_example=input_example,
-              **kwargs)
+    Model.log(
+        artifact_path=artifact_path,
+        flavor=mlflow.keras,
+        keras_model=keras_model,
+        conda_env=conda_env,
+        custom_objects=custom_objects,
+        keras_module=keras_module,
+        registered_model_name=registered_model_name,
+        signature=signature,
+        input_example=input_example,
+        await_registration_for=await_registration_for,
+        **kwargs
+    )
 
 
 def _save_custom_objects(path, custom_objects):
@@ -328,12 +402,13 @@ def _save_custom_objects(path, custom_objects):
                            :py:func:`mlflow.pyfunc.load_model`.
     """
     import cloudpickle
+
     custom_objects_path = os.path.join(path, _CUSTOM_OBJECTS_SAVE_PATH)
     with open(custom_objects_path, "wb") as out_f:
         cloudpickle.dump(custom_objects, out_f)
 
 
-def _load_model(model_path, keras_module, **kwargs):
+def _load_model(model_path, keras_module, save_format, **kwargs):
     keras_models = importlib.import_module(keras_module.__name__ + ".models")
     custom_objects = kwargs.pop("custom_objects", {})
     custom_objects_path = None
@@ -343,15 +418,26 @@ def _load_model(model_path, keras_module, **kwargs):
         model_path = os.path.join(model_path, _MODEL_SAVE_PATH)
     if custom_objects_path is not None:
         import cloudpickle
+
         with open(custom_objects_path, "rb") as in_f:
             pickled_custom_objects = cloudpickle.load(in_f)
             pickled_custom_objects.update(custom_objects)
             custom_objects = pickled_custom_objects
+
+    # If the save_format is HDF5, then we save with h5 file
+    # extension to align with prior behavior of mlflow logging
+    if save_format == "h5":
+        model_path = model_path + ".h5"
+
     from distutils.version import StrictVersion
-    if StrictVersion(keras_module.__version__.split('-')[0]) >= StrictVersion("2.2.3"):
+
+    if save_format == "h5" and StrictVersion(
+        keras_module.__version__.split("-")[0]
+    ) >= StrictVersion("2.2.3"):
         # NOTE: Keras 2.2.3 does not work with unicode paths in python2. Pass in h5py.File instead
         # of string to avoid issues.
         import h5py
+
         with h5py.File(os.path.abspath(model_path), "r") as model_path:
             return keras_models.load_model(model_path, custom_objects=custom_objects, **kwargs)
     else:
@@ -365,16 +451,23 @@ class _KerasModelWrapper:
         self._graph = graph
         self._sess = sess
 
-    def predict(self, dataframe):
+    def predict(self, data):
+        def _predict(data):
+            if isinstance(data, pd.DataFrame):
+                predicted = pd.DataFrame(self.keras_model.predict(data.values))
+                predicted.index = data.index
+            else:
+                predicted = self.keras_model.predict(data)
+            return predicted
+
         # In TensorFlow < 2.0, we use a graph and session to predict
         if self._graph is not None:
             with self._graph.as_default():
                 with self._sess.as_default():
-                    predicted = pd.DataFrame(self.keras_model.predict(dataframe.values))
+                    predicted = _predict(data)
         # In TensorFlow >= 2.0, we do not use a graph and session to predict
         else:
-            predicted = pd.DataFrame(self.keras_model.predict(dataframe.values))
-        predicted.index = dataframe.index
+            predicted = _predict(data)
         return predicted
 
 
@@ -385,16 +478,27 @@ def _load_pyfunc(path):
     :param path: Local filesystem path to the MLflow Model with the ``keras`` flavor.
     """
     import tensorflow as tf
+
     if os.path.isfile(os.path.join(path, _KERAS_MODULE_SPEC_PATH)):
         with open(os.path.join(path, _KERAS_MODULE_SPEC_PATH), "r") as f:
             keras_module = importlib.import_module(f.read())
     else:
         import keras
+
         keras_module = keras
 
+    # By default, we assume the save_format is h5 for backwards compatibility
+    save_format = "h5"
+    save_format_path = os.path.join(path, _KERAS_SAVE_FORMAT_PATH)
+    if os.path.isfile(save_format_path):
+        with open(save_format_path, "r") as f:
+            save_format = f.read()
+
+    # In SavedModel format, if we don't compile the model
+    should_compile = save_format == "tf"
     K = importlib.import_module(keras_module.__name__ + ".backend")
-    if keras_module.__name__ == "tensorflow.keras" or K.backend() == 'tensorflow':
-        if LooseVersion(tf.__version__) < LooseVersion('2.0.0'):
+    if keras_module.__name__ == "tensorflow.keras" or K.backend() == "tensorflow":
+        if LooseVersion(tf.__version__) < LooseVersion("2.0.0"):
             graph = tf.Graph()
             sess = tf.Session(graph=graph)
             # By default tf backed models depend on the global graph and session.
@@ -403,11 +507,18 @@ def _load_pyfunc(path):
             with graph.as_default():
                 with sess.as_default():  # pylint:disable=not-context-manager
                     K.set_learning_phase(0)
-                    m = _load_model(path, keras_module=keras_module, compile=False)
+                    m = _load_model(
+                        path,
+                        keras_module=keras_module,
+                        save_format=save_format,
+                        compile=should_compile,
+                    )
                     return _KerasModelWrapper(m, graph, sess)
         else:
             K.set_learning_phase(0)
-            m = _load_model(path, keras_module=keras_module, compile=False)
+            m = _load_model(
+                path, keras_module=keras_module, save_format=save_format, compile=should_compile
+            )
             return _KerasModelWrapper(m, None, None)
 
     else:
@@ -446,16 +557,25 @@ def load_model(model_uri, **kwargs):
     flavor_conf = _get_flavor_configuration(model_path=local_model_path, flavor_name=FLAVOR_NAME)
     keras_module = importlib.import_module(flavor_conf.get("keras_module", "keras"))
     keras_model_artifacts_path = os.path.join(
-        local_model_path,
-        flavor_conf.get("data", _MODEL_SAVE_PATH))
-    return _load_model(model_path=keras_model_artifacts_path, keras_module=keras_module, **kwargs)
+        local_model_path, flavor_conf.get("data", _MODEL_SAVE_PATH)
+    )
+    # For backwards compatibility, we assume h5 when the save_format is absent
+    save_format = flavor_conf.get("save_format", "h5")
+    return _load_model(
+        model_path=keras_model_artifacts_path,
+        keras_module=keras_module,
+        save_format=save_format,
+        **kwargs
+    )
 
 
 @experimental
-def autolog():
+@autologging_integration(FLAVOR_NAME)
+def autolog(log_models=True, disable=False, exclusive=False):  # pylint: disable=unused-argument
     # pylint: disable=E0611
     """
-    Enables automatic logging from Keras to MLflow. Autologging captures the following information:
+    Enables (or disables) and configures autologging from Keras to MLflow. Autologging captures
+    the following information:
 
     **Metrics** and **Parameters**
      - Training loss; validation loss; user-specified metrics
@@ -499,60 +619,84 @@ def autolog():
 
     MLflow will also log the parameters of the ``EarlyStopping`` callback,
     excluding ``mode`` and ``verbose``.
+
+    :param log_models: If ``True``, trained models are logged as MLflow model artifacts.
+                       If ``False``, trained models are not logged.
+    :param disable: If ``True``, disables the Keras autologging integration. If ``False``,
+                    enables the Keras autologging integration.
+    :param exclusive: If ``True``, autologged content is not logged to user-created fluent runs.
+                      If ``False``, autologged content is logged to the active fluent run,
+                      which may be user-created.
     """
     import keras
 
-    class __MLflowKerasCallback(keras.callbacks.Callback):
-        """
-        Callback for auto-logging metrics and parameters.
-        Records available logs after each epoch.
-        Records model structural information as params when training begins
-        """
-        def on_train_begin(self, logs=None):  # pylint: disable=unused-argument
-            try_mlflow_log(mlflow.log_param, 'num_layers', len(self.model.layers))
-            try_mlflow_log(mlflow.log_param, 'optimizer_name', type(self.model.optimizer).__name__)
-            if hasattr(self.model.optimizer, 'lr'):
-                lr = self.model.optimizer.lr if \
-                    type(self.model.optimizer.lr) is float \
-                    else keras.backend.eval(self.model.optimizer.lr)
-                try_mlflow_log(mlflow.log_param, 'learning_rate', lr)
-            if hasattr(self.model.optimizer, 'epsilon'):
-                epsilon = self.model.optimizer.epsilon if \
-                    type(self.model.optimizer.epsilon) is float \
-                    else keras.backend.eval(self.model.optimizer.epsilon)
-                try_mlflow_log(mlflow.log_param, 'epsilon', epsilon)
+    def getKerasCallback(metrics_logger):
+        class __MLflowKerasCallback(keras.callbacks.Callback, metaclass=ExceptionSafeClass):
+            """
+            Callback for auto-logging metrics and parameters.
+            Records available logs after each epoch.
+            Records model structural information as params when training begins
+            """
 
-            sum_list = []
-            self.model.summary(print_fn=sum_list.append)
-            summary = '\n'.join(sum_list)
-            tempdir = tempfile.mkdtemp()
-            try:
-                summary_file = os.path.join(tempdir, "model_summary.txt")
-                with open(summary_file, 'w') as f:
-                    f.write(summary)
-                try_mlflow_log(mlflow.log_artifact, local_path=summary_file)
-            finally:
-                shutil.rmtree(tempdir)
+            def on_train_begin(self, logs=None):  # pylint: disable=unused-argument
+                try_mlflow_log(mlflow.log_param, "num_layers", len(self.model.layers))
+                try_mlflow_log(
+                    mlflow.log_param, "optimizer_name", type(self.model.optimizer).__name__
+                )
+                if hasattr(self.model.optimizer, "lr"):
+                    lr = (
+                        self.model.optimizer.lr
+                        if type(self.model.optimizer.lr) is float
+                        else keras.backend.eval(self.model.optimizer.lr)
+                    )
+                    try_mlflow_log(mlflow.log_param, "learning_rate", lr)
+                if hasattr(self.model.optimizer, "epsilon"):
+                    epsilon = (
+                        self.model.optimizer.epsilon
+                        if type(self.model.optimizer.epsilon) is float
+                        else keras.backend.eval(self.model.optimizer.epsilon)
+                    )
+                    try_mlflow_log(mlflow.log_param, "epsilon", epsilon)
 
-        def on_epoch_end(self, epoch, logs=None):
-            if not logs:
-                return
-            try_mlflow_log(mlflow.log_metrics, logs, step=epoch)
+                sum_list = []
+                self.model.summary(print_fn=sum_list.append)
+                summary = "\n".join(sum_list)
+                tempdir = tempfile.mkdtemp()
+                try:
+                    summary_file = os.path.join(tempdir, "model_summary.txt")
+                    with open(summary_file, "w") as f:
+                        f.write(summary)
+                    try_mlflow_log(mlflow.log_artifact, local_path=summary_file)
+                finally:
+                    shutil.rmtree(tempdir)
 
-        def on_train_end(self, logs=None):
-            try_mlflow_log(log_model, self.model, artifact_path='model')
+            def on_epoch_end(self, epoch, logs=None):
+                if not logs:
+                    return
+                metrics_logger.record_metrics(logs, epoch)
 
-        # As of Keras 2.4.0, Keras Callback implementations must define the following
-        # methods indicating whether or not the callback overrides functions for
-        # batch training/testing/inference
-        def _implements_train_batch_hooks(self): return False
+            def on_train_end(self, logs=None):
+                if log_models:
+                    try_mlflow_log(log_model, self.model, artifact_path="model")
 
-        def _implements_test_batch_hooks(self): return False
+            # As of Keras 2.4.0, Keras Callback implementations must define the following
+            # methods indicating whether or not the callback overrides functions for
+            # batch training/testing/inference
+            def _implements_train_batch_hooks(self):
+                return False
 
-        def _implements_predict_batch_hooks(self): return False
+            def _implements_test_batch_hooks(self):
+                return False
+
+            def _implements_predict_batch_hooks(self):
+                return False
+
+        return __MLflowKerasCallback()
 
     def _early_stop_check(callbacks):
-        if LooseVersion(keras.__version__) < LooseVersion('2.3.0'):
+        if LooseVersion(keras.__version__) < LooseVersion("2.3.0") or LooseVersion(
+            keras.__version__
+        ) >= LooseVersion("2.4.0"):
             es_callback = keras.callbacks.EarlyStopping
         else:
             es_callback = keras.callbacks.callbacks.EarlyStopping
@@ -563,15 +707,14 @@ def autolog():
 
     def _log_early_stop_callback_params(callback):
         if callback:
-            try:
-                earlystopping_params = {'monitor': callback.monitor,
-                                        'min_delta': callback.min_delta,
-                                        'patience': callback.patience,
-                                        'baseline': callback.baseline,
-                                        'restore_best_weights': callback.restore_best_weights}
-                try_mlflow_log(mlflow.log_params, earlystopping_params)
-            except Exception:  # pylint: disable=W0703
-                return
+            earlystopping_params = {
+                "monitor": callback.monitor,
+                "min_delta": callback.min_delta,
+                "patience": callback.patience,
+                "baseline": callback.baseline,
+                "restore_best_weights": callback.restore_best_weights,
+            }
+            try_mlflow_log(mlflow.log_params, earlystopping_params)
 
     def _get_early_stop_callback_attrs(callback):
         try:
@@ -579,70 +722,74 @@ def autolog():
         except Exception:  # pylint: disable=W0703
             return None
 
-    def _log_early_stop_callback_metrics(callback, history):
+    def _log_early_stop_callback_metrics(callback, history, metrics_logger):
         if callback:
             callback_attrs = _get_early_stop_callback_attrs(callback)
             if callback_attrs is None:
                 return
             stopped_epoch, restore_best_weights, patience = callback_attrs
-            try_mlflow_log(mlflow.log_metric, 'stopped_epoch', stopped_epoch)
+            metrics_logger.record_metrics({"stopped_epoch": stopped_epoch})
             # Weights are restored only if early stopping occurs
             if stopped_epoch != 0 and restore_best_weights:
                 restored_epoch = stopped_epoch - max(1, patience)
-                try_mlflow_log(mlflow.log_metric, 'restored_epoch', restored_epoch)
-                restored_metrics = {key: history.history[key][restored_epoch]
-                                    for key in history.history.keys()}
+                metrics_logger.record_metrics({"restored_epoch": restored_epoch})
+                restored_index = history.epoch.index(restored_epoch)
+                restored_metrics = {
+                    key: history.history[key][restored_index] for key in history.history.keys()
+                }
                 # Checking that a metric history exists
                 metric_key = next(iter(history.history), None)
                 if metric_key is not None:
                     last_epoch = len(history.history[metric_key])
-                    try_mlflow_log(mlflow.log_metrics, restored_metrics, step=last_epoch)
+                    metrics_logger.record_metrics(restored_metrics, last_epoch)
 
     def _run_and_log_function(self, original, args, kwargs, unlogged_params, callback_arg_index):
-        if not mlflow.active_run():
-            try_mlflow_log(mlflow.start_run)
-            auto_end_run = True
-        else:
-            auto_end_run = False
-
         log_fn_args_as_params(original, args, kwargs, unlogged_params)
         early_stop_callback = None
 
         # Checking if the 'callback' argument of the function is set
-        if len(args) > callback_arg_index:
-            tmp_list = list(args)
-            early_stop_callback = _early_stop_check(tmp_list[callback_arg_index])
-            tmp_list[callback_arg_index] += [__MLflowKerasCallback()]
-            args = tuple(tmp_list)
-        elif 'callbacks' in kwargs:
-            early_stop_callback = _early_stop_check(kwargs['callbacks'])
-            kwargs['callbacks'] += [__MLflowKerasCallback()]
-        else:
-            kwargs['callbacks'] = [__MLflowKerasCallback()]
+        run_id = mlflow.active_run().info.run_id
+        with batch_metrics_logger(run_id) as metrics_logger:
+            mlflowKerasCallback = getKerasCallback(metrics_logger)
+            if len(args) > callback_arg_index:
+                tmp_list = list(args)
+                early_stop_callback = _early_stop_check(tmp_list[callback_arg_index])
+                tmp_list[callback_arg_index] += [mlflowKerasCallback]
+                args = tuple(tmp_list)
+            elif kwargs.get("callbacks"):
+                early_stop_callback = _early_stop_check(kwargs["callbacks"])
+                kwargs["callbacks"] += [mlflowKerasCallback]
+            else:
+                kwargs["callbacks"] = [mlflowKerasCallback]
 
-        _log_early_stop_callback_params(early_stop_callback)
+            try_mlflow_log(_log_early_stop_callback_params, early_stop_callback)
 
-        history = original(self, *args, **kwargs)
+            history = original(self, *args, **kwargs)
 
-        _log_early_stop_callback_metrics(early_stop_callback, history)
-
-        if auto_end_run:
-            try_mlflow_log(mlflow.end_run)
+            try_mlflow_log(
+                _log_early_stop_callback_metrics, early_stop_callback, history, metrics_logger
+            )
 
         return history
 
-    @gorilla.patch(keras.Model)
-    def fit(self, *args, **kwargs):
-        original = gorilla.get_original_attribute(keras.Model, 'fit')
-        unlogged_params = ['self', 'x', 'y', 'callbacks', 'validation_data', 'verbose']
+    def fit(original, self, *args, **kwargs):
+        unlogged_params = ["self", "x", "y", "callbacks", "validation_data", "verbose"]
         return _run_and_log_function(self, original, args, kwargs, unlogged_params, 5)
 
-    @gorilla.patch(keras.Model)
-    def fit_generator(self, *args, **kwargs):
-        original = gorilla.get_original_attribute(keras.Model, 'fit_generator')
-        unlogged_params = ['self', 'generator', 'callbacks', 'validation_data', 'verbose']
+    def fit_generator(original, self, *args, **kwargs):
+        """
+        NOTE: `fit_generator()` is deprecated in Keras >= 2.4.0 and simply wraps `fit()`.
+        To avoid unintentional creation of nested MLflow runs caused by a patched
+        `fit_generator()` method calling a patched `fit()` method, we only patch
+        `fit_generator()` in Keras < 2.4.0.
+        """
+        unlogged_params = ["self", "generator", "callbacks", "validation_data", "verbose"]
         return _run_and_log_function(self, original, args, kwargs, unlogged_params, 4)
 
-    settings = gorilla.Settings(allow_hit=True, store_hit=True)
-    gorilla.apply(gorilla.Patch(keras.Model, 'fit', fit, settings=settings))
-    gorilla.apply(gorilla.Patch(keras.Model, 'fit_generator', fit_generator, settings=settings))
+    safe_patch(FLAVOR_NAME, keras.Model, "fit", fit, manage_run=True)
+    # `fit_generator()` is deprecated in Keras >= 2.4.0 and simply wraps `fit()`.
+    # To avoid unintentional creation of nested MLflow runs caused by a patched
+    # `fit_generator()` method calling a patched `fit()` method, we only patch
+    # `fit_generator()` in Keras < 2.4.0.
+    if LooseVersion(keras.__version__) < LooseVersion("2.4.0"):
+        safe_patch(FLAVOR_NAME, keras.Model, "fit_generator", fit_generator, manage_run=True)
